<?php defined('_JEXEC') or die;
/**
 * @version    1.0.0
 * @package    com_jmailq
 * @author     Alex Chartier <alex@skurvishenterprises.com>
 * @copyright  2018 Skurvish Enterprises
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

use Joomla\CMS\Application\AdministratorApplication;
use Joomla\CMS\Factory;
use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Installer\InstallerScriptInterface;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Version;
use Joomla\DI\Container;
use Joomla\DI\ServiceProviderInterface;
use Joomla\Filesystem\File;
use Joomla\Filesystem\Folder;

/* This simple plugin is used to update the fabrik updtesite and changelog url, it is only useful pre fabrik 4.3 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

return new class () implements ServiceProviderInterface {
    public function register(Container $container)
    {
        $container->set(InstallerScriptInterface::class, new class ($container->get(AdministratorApplication::class)) implements InstallerScriptInterface {

            public $app = null;

            public function __construct(AdministratorApplication $app)
            {
                $this->app = $app;
            }

            public function postFlight(string $type, InstallerAdapter $parent): bool
            {
                return true;
            }

            public function install(InstallerAdapter $parent) : bool {
                return true;
            }

            public function uninstall(InstallerAdapter $parent) : bool {
                return true;
            }

            public function update(InstallerAdapter $parent) : bool {
                return true;
            }

            public function preflight($type, InstallerAdapter $parent) : bool {
                /* First check the installed version */
                $db = Factory::getContainer()->get('DatabaseDriver');
                $query = $db->getQuery(true);
                $query->select('*')->from('#__extensions')->where('element="pkg_fabrik"')->where('type="package"');
                $db->setQuery($query);
                $row = $db->loadObject();
                if (empty($row)) {
                    throw new RuntimeException('There is no compatabile Fabrik package installed!');
                    return false;
                }
                $manifest_cache = json_decode($row->manifest_cache);
                if (empty($manifest_cache)) {
                    throw new RuntimeException('Unable to locate Fabrik manifest cache! Cannot determine installed version!');
                    return false;
                }
                if (version_compare($manifest_cache->version, '4.2', '>=')) {
                    throw new RuntimeException('This cannot be installed on versions 4.2 or higher!');
                    return false;
                }

                $changelogurl = htmlentities("https://fabrikar.com/index.php?option=com_ochsubscriptions&view=changelog&format=xml&element=fabrikbase");
                $updateserverurl = htmlentities("https://fabrikar.com/index.php?option=com_ochsubscriptions&amp;view=updater&amp;format=xml&amp;element=fabrikbase");

                /* First let's update the pgk_xml files */
                $path = JPATH_ADMINISTRATOR . '/manifests/packages/';
                $manifests = Folder::Files($path, '.xml');
                /* Filter out anything that is not fabrik */
                $manifests = array_filter($manifests, function ($v) {
                    return (strpos($v, 'pkg_fabrik') === 0);
                });

                /* Now update each that we found */
                foreach ($manifests as $manifest) {
                    $file = $path . $manifest;
                    $xml = simplexml_load_file($file);
                    /* Set the dlid to what we want */
                    unset($xml->dlid);
                    $xml->addChild('dlid');
                    $dlid = $xml->dlid;
                    $dlid->addAttribute('prefix', 'key=');
                    $dlid->addAttribute('suffix', '');
                    /* Set the changelog to what we want */
                    $xml->changelogurl = $changelogurl;
                    /* Set the updateserver */
                    $xml->updateservers->server = $updateserverurl;
                    /* Save it back */
                    $xml->asXML($file); 
                }

                /* Update the changelog url in the extensions table */
                $query = $db->getQuery(true);
                $query
                    ->select('*')
                    ->from('#__extensions')
                    ->where($db->quoteName('type') . ' = ' . $db->quote('package'))
                    ->where($db->quoteName('element') . ' like("%pkg_fabrik%")');
                $db->setQuery($query);
                $packages = $db->loadObjectList();
                foreach($packages as $package) { 
                    $package->changelogurl = $changelogurl;
                    if (empty($package->manifest_cache) || $package->manifest_cache == 'false') {
                        /* For some reason on some installs/upgrades the manifest cache shows as false, we will just assume a 4.0 manifest and fill it in */
                        $package->manifest_cache = '{"name":"Fabrik Base Package","type":"package","creationDate":"February 2024","author":"Fabrikar","copyright":"Copyright (C) 2005-2024 Fabrikar - All rights reserved.","authorEmail":"","authorUrl":"","version":"4.0","description":"Fabrik Base Package","group":"","filename":"pkg_fabrik"}';
                    }
                    /* Store the extension back */
                    $db->updateObject('#__extensions', $package, 'extension_id');
                }

                /* Update the updatesite url in the updatesites table */
                /* Note: A rebuild of the update sites would accomplish this, but why make the user perform an extra step */
                $query
                    ->clear()
                    ->select('*')
                    ->from('#__update_sites')
                    ->where('name like ("%fabrik%")')
                    ->where('type="extension"')
                    ->where('location like("%element=Fabrik%")');
                $db->setQuery($query);
                $sites = $db->loadObjectList();
                if (empty($sites)) {
                    $site = (object)['update_site_id'=>null, 'name'=>'Fabrik', 'type'=>'extension', 'enabled'=>1, 'location'=>$updateserverurl, 'extra_query'=>'key='];
                    $db->insertObject('#__update_sites', $site, 'update_site_id');
                } else {
                    foreach ($sites as $site) {
                        $site->location = $updateserverurl;
                        /* Check if there is already a download key set, if not set the extra query string */
                        $xqLen = strlen($site->extra_query);
                        if ($xqLen == 0 || $xqLen < 5) {
                            $site->extra_query = 'key=';
                        }
                        $db->updateObject('#__update_sites', $site, 'update_site_id');
                    }
                }

                Factory::getApplication()->enqueueMessage("Updates complete.<br />You will see an installation failure message below.<br />You can safely ignore this message, this plugin is never intended to actually install.<br />Note: Please go to System->Update Sites in your admin and click on Rebuild to clean up old update sites.<br />Also note: You will need to insert your download ID in the Fabrik Update Site entry.");
                return false;
            }

        });
    }
};

